@@ -1,6 +1,9 @@
 import os
 import datetime
 import re
+import traceback
+
+import pandas as pd
 import subprocess
 from time import sleep
 from typing import Tuple
@@ -23,6 +26,7 @@ from selenium.webdriver.firefox.service import Service
 
 from .common_config import *
 from .smb_functions import *
+from .smb_functions_manager import SmbFunctionsManager
 
 
 # from selenium.webdriver.firefox.firefox_profile import FirefoxProfile
@@ -106,7 +110,7 @@ class WebSapCommonFunctions:
     def initial_browser(self, has_save_folder: bool = False, save_folder_path: str = '', has_proxy: bool = True, proxy_area: str = 'hk',
                         is_private: bool = False, is_headless=False, firefox_binary_location: str = '', geckodriver_binary_location: str = '',
                         timeout=1800, auto_get_screenshot=False, time_interval=15, browser_screenshot_tag='', remote_save=False, username='', password='', server_name='',
-                        share_name='', remote_folder_path=''):
+                        share_name='', remote_folder_path='', scale_ratio=1):
         """This function is used to initial FireFox browser
 
         Args:
@@ -128,6 +132,7 @@ class WebSapCommonFunctions:
             server_name(str):This is the server name (url), e.g. szh0fs06.apac.bosch.com
             share_name(str): This is the share_name of public folder, e.g. GS_ACC_CN$
             remote_folder_path(str): This is the public folder path that file will be saved under share name folder
+            scale_ratio(float): This is the scale ratio of browser interface, e.g. 0.8 = 80%, 1.0 = 100%, 1.25 = 125%
         Returns:
             Tuple[Firefox,WebDriverWait]: [self.browser, self.wait]
 
@@ -148,6 +153,9 @@ class WebSapCommonFunctions:
         profile_dir = tempfile.mkdtemp(prefix="ffprof_")
         opts.set_preference("profile", profile_dir)
 
+        # ⚡ 设置缩放比例 (0.8 = 80%，1.0 = 100%，1.25 = 125%)
+        opts.set_preference("layout.css.devPixelsPerPx", f"{scale_ratio}")
+
         # opts.headless = is_headless
         if is_headless:
             opts.add_argument("--headless")
@@ -221,6 +229,8 @@ class WebSapCommonFunctions:
         for file_name in os.listdir(screenshot_folder_path):
             os.remove(screenshot_folder_path + os.sep + file_name)
 
+        smb_client = SmbFunctionsManager(username, password, server_name, share_name)
+
         while not self.stop_screenshot_thread:
             if time_interval < 5:
                 sleep(5)
@@ -239,21 +249,32 @@ class WebSapCommonFunctions:
                 sleep(1)
                 if remote_save:
                     remote_screenshot_folder_path = remote_folder_path + os.sep + 'Screenshots'
-                    is_folder_exist = smb_check_folder_exist(username, password, server_name, share_name, remote_screenshot_folder_path)
+                    # is_folder_exist = smb_check_folder_exist(username, password, server_name, share_name, remote_screenshot_folder_path)
+                    is_folder_exist = smb_client.smb_check_folder_exist(remote_screenshot_folder_path)
                     if not is_folder_exist:
-                        smb_create_folder(username, password, server_name, share_name, remote_screenshot_folder_path)
+                        # smb_create_folder(username, password, server_name, share_name, remote_screenshot_folder_path)
+                        smb_client.smb_create_folder(remote_screenshot_folder_path)
 
                     if browser_screenshot_tag:
                         remote_screenshot_folder_path = remote_screenshot_folder_path + os.sep + browser_screenshot_tag
-                        is_folder_exist = smb_check_folder_exist(username, password, server_name, share_name, remote_screenshot_folder_path)
+                        # is_folder_exist = smb_check_folder_exist(username, password, server_name, share_name, remote_screenshot_folder_path)
+                        is_folder_exist = smb_client.smb_check_folder_exist(remote_screenshot_folder_path)
                         if not is_folder_exist:
-                            smb_create_folder(username, password, server_name, share_name, remote_screenshot_folder_path)
+                            # smb_create_folder(username, password, server_name, share_name, remote_screenshot_folder_path)
+                            smb_client.smb_create_folder(remote_screenshot_folder_path)
 
                     remote_file_path = remote_screenshot_folder_path + os.sep + screenshot_file_path.split(os.sep)[-1]
-                    smb_copy_file_local_to_remote(username, password, server_name, share_name, screenshot_file_path, remote_file_path)
+                    # smb_copy_file_local_to_remote(username, password, server_name, share_name, screenshot_file_path, remote_file_path)
+                    smb_client.smb_copy_file_local_to_remote(screenshot_file_path, remote_file_path)
             except InvalidSessionIdException:
+                if smb_client:
+                    smb_client.close_smb_connection()
+                print('[SCREENSHOT ERROR]:', traceback.format_exc())
                 break
 
+        if smb_client:
+            smb_client.close_smb_connection()
+
     def wait_page_loaded(self):
         """This function is used to wait page loaded
 
@@ -346,7 +367,8 @@ class WebSapCommonFunctions:
                     return True
             else:
                 sleep(self.time_interval + 1)  # auto get one more screenshot
-                self.stop_screenshot_thread = True
+                # self.stop_screenshot_thread = True
+                print('[LOGIN ERROR]: Please check login error!')
                 return False
 
     def login_sap_with_password(self, sap_user: str = '', sap_password: str = ''):
@@ -1042,6 +1064,89 @@ class WebSapCommonFunctions:
 
         return False
 
+    def input_multiple_selection_with_field_label(self,
+                                                  value_list: list,
+                                                  field_label: str = '',
+                                                  tab_index: int = 1,
+                                                  input_css_path: str = MULTIPLE_WINDOW_FIRST_INPUT_CSS_SELECTOR,
+                                                  clear_section_data: bool = False,
+                                                  get_screenshot: bool = False,
+                                                  screenshot_folder_path: str = '',
+                                                  screenshot_file_name_tag: str = 'error_info',
+                                                  name_format: str = 'time',
+                                                  paste_method: str = 'clipboard',
+                                                  by: By = By.CSS_SELECTOR
+                                                  ):
+
+        """This function is used to input multiple selection for company code, gl account and so on
+
+        Args:
+            paste_method(str): This is the copy and paste method for multiple input. e.g. clipboard or file
+            tab_index(int): This is the tab index of multiple selection
+            field_label(int): This is the name of Multiple selection item
+            input_css_path(str): This is the css selector to locate first input element
+            value_list(list): This is the values need to be input
+            clear_section_data(bool): This is the indicator whether delete all section data before pasting new values
+            get_screenshot(bool): This is the flag that indicates whether to save screenshot
+            screenshot_file_name_tag(str):This is the first part file name of screenshot
+            screenshot_folder_path(str): This is the folder path of screenshot
+            name_format(str): This indicates whether user date or datetime to name screenshot
+            by(By): This is the By method to find element
+        """
+        self.wait_element_presence_by_css_selector("div[title^='Multiple selection' i]")
+
+        all_rli_elements = self.browser.find_elements(By.CSS_SELECTOR, "div[ct='RLI']")
+
+        field_label = field_label.strip()
+        start_search_button = False
+
+        for rli_element in all_rli_elements:
+            try:
+                label_element = rli_element.find_element(By.CSS_SELECTOR, "span.lsLabel__text")
+                label_text = label_element.text.strip()
+                if label_text == field_label:
+                    start_search_button = True
+            except:
+                try:
+                    button_element = rli_element.find_element(By.CSS_SELECTOR, "div[title^='Multiple selection' i]")
+                    if start_search_button:
+                        self.move_to_and_click_element('', target_element=button_element, by=by)
+                        sleep(2)
+                        self.wait_invisibility_of_loading_window()
+                        self.wait.until(EC.presence_of_element_located((by, input_css_path)))
+                        if clear_section_data:
+                            self.wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, DELETE_ENTIRE_SELECTION_BUTTON_CSS_SELECTOR)))
+                            sleep(1)
+                            self.click_or_input_by_css_selector(DELETE_ENTIRE_SELECTION_BUTTON_CSS_SELECTOR, 'click')
+                            sleep(2)
+                        if tab_index == 2:
+                            sleep(3)
+                            self.click_or_input_by_css_selector(SELECT_RANGES_CSS_SELECTOR, 'click')
+                        elif tab_index == 3:
+                            sleep(3)
+                            self.click_or_input_by_css_selector(EXCLUDE_SINGLE_VALUES_CSS_SELECTOR, 'click')
+                        elif tab_index == 4:
+                            sleep(3)
+                            self.click_or_input_by_css_selector(EXCLUDE_RANGES_CSS_SELECTOR, 'click')
+
+                        self.wait_multiple_selection_switch(tab_index)
+                        self.wait.until(EC.presence_of_element_located((by, input_css_path)))
+                        if paste_method == 'clipboard':
+                            second_paste = tab_index in [2, 4]
+                            self.copy_paste_values(value_list, input_css_path, second_paste=second_paste, by=by)
+
+                        if get_screenshot:
+                            self.get_screenshot(screenshot_folder_path, screenshot_file_name_tag, name_format)
+                            sleep(1)
+                        self.click_or_input_by_css_selector(COPY_BUTTON_CSS_SELECTOR, 'click')
+                        self.wait.until(EC.invisibility_of_element_located((by, input_css_path)))
+                        return True
+
+                except:
+                    pass
+
+        return False
+
     def input_query_multiple_selection_with_index(self,
                                                   value_list: list,
                                                   multiple_selection_index: int = 1,
@@ -1075,7 +1180,8 @@ class WebSapCommonFunctions:
         multiple_selection_buttons = self.browser.find_elements(By.CSS_SELECTOR, "div[title^='Multiple selection' i]")
         for button_index, multiple_selection_button in enumerate(multiple_selection_buttons):
             if button_index + 1 == multiple_selection_index:
-                self.move_to_and_click_element(f"tr[id$='mrss-cont-left-Row-{button_index}'] td div div.urST5SCMetricInner")
+                # self.move_to_and_click_element(f"tr[id$='mrss-cont-left-Row-{button_index}'] td div div.urST5SCMetricInner")
+                self.move_to_and_click_element(f"tr[iidx='{button_index}'] td div div.urST5SCMetricInner")
                 sleep(2)
                 # self.click_or_input_by_element(multiple_selection_button, 'click')
                 self.move_to_and_click_element('', target_element=multiple_selection_button, by=by)
@@ -1090,7 +1196,7 @@ class WebSapCommonFunctions:
                 # click select ranges tab
                 if tab_index == 2:
                     sleep(3)
-                    self.click_or_input_by_css_selector(SELECT_RANGES_CSS_SELECTOR, 'click')
+                    self.click_or_input_by_css_selector(SELECT_RANGES_QUERY_CSS_SELECTOR, 'click')
                 elif tab_index == 3:
                     sleep(3)
                     self.click_or_input_by_css_selector(EXCLUDE_SINGLE_VALUES_CSS_SELECTOR, 'click')
