@@ -1,14 +1,7 @@
 import io
 import os
-import uuid
 import smbclient
 import pandas as pd
-from smbprotocol.connection import Connection
-from smbprotocol.session import Session
-from smbprotocol.tree import TreeConnect
-from smbprotocol.open import Open
-from smbprotocol.file_info import FileInformationClass
-from smbprotocol.exceptions import SMBResponseException, NtStatus
 
 
 def smb_copy_file_local_to_remote(username, password, server_name, share_name, local_file_path, remote_file_path, port=445):
@@ -150,63 +143,26 @@ def smb_traverse_remote_folder(username, password, server_name, share_name, remo
         remote_folder_path(str): This is the public folder path that folder will be saved under share name folder
         port(int): This is the port number of the server name
     """
-
     traverse_result_list = []
-    client_guid = uuid.uuid4()
-    connection = Connection(guid=client_guid, server_name=server_name, port=port)
-    connection.connect()
+    connection_cache = {}
+    full_remote_folder_path = f"//{server_name}/{share_name}/{remote_folder_path}".rstrip("/")
 
-    session = Session(connection, username, password)
-    session.connect()
-
-    tree = TreeConnect(session, f"\\\\{server_name}\\{share_name}")
-    tree.connect()
-
-    folder_open = Open(tree, remote_folder_path)
-    folder_open.create(impersonation_level=0,
-                       desired_access=0x00000001,  # read only
-                       file_attributes=0,
-                       share_access=0x00000001,  # allow other process
-                       create_disposition=1,  # OPEN_EXISTING
-                       create_options=0
-                       )
+    smbclient.register_session(server_name, username=username, password=password, port=port, encrypt=True, connection_cache=connection_cache)
     try:
-        pattern = "*"  # match all
-        file_information_class = FileInformationClass.FILE_DIRECTORY_INFORMATION
-        # file_info = folder_open.query_directory(pattern, file_information_class)
-
-        all_file_info = []
-        while True:
-            try:
-                batch = folder_open.query_directory(pattern, file_information_class)
-                if not batch:
-                    break
-                all_file_info.extend(batch)
-            except SMBResponseException as e:
-                if e.status == NtStatus.STATUS_NO_MORE_FILES:
-                    break
-                raise
-
-        for item in all_file_info:
-            item_name = item.fields.get('file_name').value.decode('utf-16')
-            if item_name not in ['.', '..']:
-                creation_time = item.fields.get('creation_time').value
-                last_access_time = item.fields.get('last_access_time').value
-                last_write_time = item.fields.get('last_write_time').value
-                change_time = item.fields.get('change_time').value
-
-                if item.fields.get('file_attributes').value & 0x10:  # check whether item is folder
-                    traverse_result_list.append({'name': item_name,
-                                                 'is_folder': True, 'is_file': False, 'creation_time': creation_time, 'last_access_time': last_access_time,
-                                                 'last_write_time': last_write_time, 'change_time': change_time})
-                else:
-                    traverse_result_list.append({'name': item_name,
-                                                 'is_folder': False, 'is_file': True, 'creation_time': creation_time, 'last_access_time': last_access_time,
-                                                 'last_write_time': last_write_time, 'change_time': change_time})
+        for entry in smbclient.scandir(full_remote_folder_path, connection_cache=connection_cache):
+            if entry.name not in [".", ".."]:
+                stat_info = entry.stat()
+                traverse_result_list.append({
+                    "name": entry.name,
+                    "is_folder": entry.is_dir(),
+                    "is_file": entry.is_file(),
+                    "creation_time": stat_info.st_ctime,
+                    "last_access_time": stat_info.st_atime,
+                    "last_write_time": stat_info.st_mtime,
+                    "change_time": stat_info.st_mtime,
+                })
     finally:
-        connection.disconnect()
-        session.disconnect()
-        tree.disconnect()
+        smbclient.reset_connection_cache(connection_cache=connection_cache)
 
     return traverse_result_list
 
